<#
#### requires active directory powershell module on Windows Server####
<#
.SYNOPSIS
shows exchange schema version, objectVersion (Default),objectVersion (Configuration)
https://docs.microsoft.com/en-us/exchange/plan-and-deploy/prepare-ad-and-domains?view=exchserver-2019
.DESCRIPTION

.PARAMETER 
no parameters required
.INPUTS
none
.OUTPUTS

.NOTES
   Version:        0.1
   Author:         Alexander Koehler
   Creation Date:  Thursday, April 11th 2019, 8:25:41 pm
   File: ad-schema-version-exchange-0.1.ps1
   Copyright (c) 2019 blog.it-koehler.com
HISTORY:
Date      	          By	Comments
----------	          ---	----------------------------------------------------------

.LINK
   https://blog.it-koehler.com/en/

.COMPONENT
 Required Modules: ActiveDirectory Powershell (RSAT)

.LICENSE
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the Software), to deal
in the Software without restriction, including without limitation the rights
to use copy, modify, merge, publish, distribute sublicense and /or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 
.EXAMPLE
change to the directory where script is stored then you can execute it with .\ad-schema-version-exchange-0.1.ps1
#>

#check if Powershell module is installed 
if((Get-WindowsFeature RSAT-AD-PowerShell).Installed){
   
  #get active directory information
  $schemaPath = (Get-ADRootDSE).schemaNamingContext
  #get exchange schema version
  Function Get-ExchangeUpperRange
  {
  
   Param(
     [parameter(Mandatory=$true)]
     [ValidateNotNullOrEmpty()]
     [Alias("Schemapath")]
     [String[]]
     $RootDSE
  
  )
  # additional information see https://eightwone.com/references/schema-versions/
  $schemaPath = "$RootDSE"
  $schemaobj = (Get-ADObject -filter * -SearchBase $schemaPath -Properties * | Where-Object {$_.Name -eq "ms-Exch-Schema-Version-Pt"})
  $rangeupper = $schemaobj.rangeUpper
  Write-Output "Exchange Schemaversion: $rangeupper"
  }
  #get exchange object version configuration  
  Function Get-ExchangeObjConf
  {
  $domain = [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain() 
  $distname = $domain.GetDirectoryEntry().distinguishedName 
  $config = [ADSI]"LDAP://CN=Microsoft Exchange,CN=Services,CN=Configuration,$distname" 
  $orgName = $config.psbase.children | Where-Object {$_.objectClass -eq 'msExchOrganizationContainer'} 
  $ldappath = (($orgName).Path)
  $objconfig = ([ADSI]"$ldappath")
  $objconfigver = $objconfig.objectVersion
  Write-Output "Exchange objectVersion (Configuration): $objconfigver"
  }
    #get exchange object configuration (default )  
  Function Get-ExchangeObjDefaultConf
  {
  $domain = [System.DirectoryServices.ActiveDirectory.Domain]::GetCurrentDomain() 
  $distname = $domain.GetDirectoryEntry().distinguishedName 
  $objdefault = ([ADSI]"LDAP://CN=Microsoft Exchange System Objects,$distname")
  $objdefaultver =$objdefault.objectVersion
  Write-Output "Exchange objectVersion (Default): $objdefaultver"
  }
  Write-Host "### Exchange Active Directory versions ###"
  Get-ExchangeUpperRange -RootDSE "$schemaPath"
  Get-ExchangeObjDefaultConf
  Get-ExchangeObjConf 
  Write-Host " "
  Write-Host "###### additional links:" -ForegroundColor Yellow
  Write-Host "See also this link to check your schema version EX2019: https://docs.microsoft.com/en-us/exchange/plan-and-deploy/prepare-ad-and-domains?view=exchserver-2019#exchange-active-directory-versions" -ForegroundColor Yellow
  Write-Host "See also this link to check your schema version EX2016:https://docs.microsoft.com/en-us/exchange/plan-and-deploy/prepare-ad-and-domains?view=exchserver-2016#exchange-active-directory-versions" -ForegroundColor Yellow
  pause
  
  }
  else{
  
    throw "no active directory powershell module detected please check RSAT Tools 'Get-WindowsFeature RSAT-AD-PowerShell' please install RSAT Tools first on this machine before executing this script
    Import-Module ServerManager
    Add-WindowsFeature -Name 'RSAT-AD-PowerShell' -IncludeAllSubFeature "
    pause
  
  }  